@@ -2,6 +2,7 @@ TAopt <- function(OF, algo = list(), ...) {
     algoD <- list(nD = 2000L, ## random steps for computing thresholds
                   nT = 10L,   ## number of thresholds
                   nS = 1000L, ## steps per threshold
+                  nI = NULL,  ## total number of iterations
                   q = 0.5,    ## starting quantile for thresholds
                   x0 = NULL,  ## initial solution
                   vT = NULL,  ## threshold sequence
@@ -13,14 +14,19 @@ TAopt <- function(OF, algo = list(), ...) {
                   storeF = TRUE,
                   storeSolutions = FALSE,
                   classify = FALSE,
-                  OF.target = NULL)
+                  OF.target = NULL,
+                  thresholds.only = FALSE)
 
     checkList(algo, algoD)
     algoD[names(algo)] <- algo
-    if (!exists(".Random.seed", envir = .GlobalEnv,
-                inherits = FALSE))
-        state <- NA else state <- .Random.seed
+    state <- if (!exists(".Random.seed", envir = .GlobalEnv,
+                         inherits = FALSE))
+                 NA
+             else
+                 .Random.seed
 
+    if (!is.null(algoD$nI))
+        algoD$nS <- ceiling(algoD$nI/algoD$nT)
 
     ## user *must* specify the following
     if (is.null(algoD$neighbour))
@@ -116,6 +122,19 @@ TAopt <- function(OF, algo = list(), ...) {
     nT <- length(vT)
     niter <- nS * nT
 
+    if (algoD$thresholds.only) {
+        ans <- list(xbest = NA,
+                    OFvalue = NA,
+                    Fmat = NA,
+                    xlist = NA,
+                    vT = vT,
+                    initial.state = state,
+                    x0 = x0)
+        if (algoD$classify)
+            class(ans) <- "TAopt"
+        return(ans)
+    }
+    
     ## evaluate initial solution
     xc <- x0
     xcF <- OF1(xc)
@@ -237,7 +256,7 @@ TA.info <- function(n = 0L) {
 }
 
 
-                                        # METHODS
+        # METHODS
 
 print.TAopt <- function(x, ...) {
     nt <- length(x$vT)
